var Immutable, callUpdater, core, getStore, recorderEmit, recorderListeners;

Immutable = require("immutable");

core = Immutable.Map({
  records: Immutable.List(),
  pointer: 0,
  isTravelling: false,
  initial: Immutable.Map(),
  store: Immutable.Map(),
  updater: function(state) {
    return state;
  },
  inProduction: false
});

recorderListeners = Immutable.List();

recorderEmit = function() {
  return recorderListeners.forEach(function(fn) {
    return fn(core);
  });
};

callUpdater = function(actionType, actionData) {
  var backward, chunks, groupName, nextPointer, updater;
  chunks = actionType.split("/");
  groupName = chunks[0];
  updater = function(acc, action) {
    return core.get('updater')(acc, action.get(0), action.get(1));
  };
  if (groupName === "actions-recorder") {
    switch (chunks[1]) {
      case "commit":
        return {
          initial: core.get('records').reduce(updater, core.get('initial')),
          records: Immutable.List(),
          pointer: 0,
          isTravelling: false
        };
      case "reset":
        return {
          records: Immutable.List(),
          pointer: 0,
          isTravelling: false
        };
      case "peek":
        return {
          pointer: actionData,
          isTravelling: true
        };
      case "run":
        return {
          isTravelling: !core.get('isTravelling'),
          pointer: 0
        };
      case "merge-before":
        if (core.get('pointer') === 0) {
          return {};
        } else {
          return {
            initial: core.get('records').slice(0, core.get('pointer') - 1).reduce(updater, core.get('initial')),
            records: core.get('records').slice(core.get('pointer') - 1),
            pointer: 1
          };
        }
        break;
      case "clear-after":
        return {
          records: core.get('records').slice(0, core.get('pointer'))
        };
      case 'step':
        backward = actionData;
        if (backward) {
          if (core.get('pointer') > 0) {
            nextPointer = core.get('pointer') - 1;
          } else {
            nextPointer = core.get('pointer');
          }
        } else {
          if (core.get('pointer') < core.get('records').size) {
            nextPointer = core.get('pointer') + 1;
          } else {
            nextPointer = core.get('pointer');
          }
        }
        return {
          pointer: nextPointer
        };
      default:
        console.warn("Unknown actions-recorder action: " + actionType);
        return {};
    }
  } else {
    return {
      records: core.get('records').push(Immutable.List([actionType, actionData]))
    };
  }
};

getStore = function() {
  var updater;
  updater = function(acc, action) {
    return core.get('updater')(acc, action.get(0), action.get(1));
  };
  if (core.get('isTravelling') && core.get('pointer') >= 0) {
    return core.get('records').slice(0, core.get('pointer')).reduce(updater, core.get('initial'));
  } else {
    return core.get('records').reduce(updater, core.get('initial'));
  }
};

exports.setup = function(options) {
  core = core.merge(Immutable.fromJS(options));
  core = core.set('store', core.get('initial'));
  if (core.get('inProduction')) {
    return setInterval(function() {
      if (core.get('records').size > 400 && (!core.get('isTravelling'))) {
        return exports.dispatch('actions-recorder/commit');
      }
    }, 10 * 60 * 1000);
  }
};

exports.hotSetup = function(options) {
  var code;
  core = core.merge(Immutable.fromJS(options));
  code = core.set('store', getStore());
  return recorderEmit();
};

exports.request = function(fn) {
  return fn(core);
};

exports.getState = function() {
  return exports.getStore();
};

exports.getStore = function() {
  return core.get('store');
};

exports.subscribe = function(fn) {
  return recorderListeners = recorderListeners.unshift(fn);
};

exports.unsubscribe = function(fn) {
  return recorderListeners = recorderListeners.filterNot(function(listener) {
    return listener === fn;
  });
};

exports.dispatch = function(actionType, actionData) {
  actionData = Immutable.fromJS(actionData);
  core = core.merge(callUpdater(actionType, actionData));
  core = core.set('store', getStore());
  return recorderEmit();
};
